from pypika import Query, Table, Columns, Field, PostgreSQLQuery
from pypika.functions import Now
from ..custom_functions import Timezone


class ECardPurchasesStg(Table):

    def __init__(self, schema):
        super().__init__(name = 'e_card_purchases_stg', schema = schema)

        self.__columns__ = Columns(
            ('dwh_updated_at_myt',"timestamp default timezone('Asia/Taipei', now())"),
            ('country','text'),
            ('schema_name','varchar(300)'),
            ('id','serial primary key'),
            ('company_id','int4'),
            ('company_name','text'),
            ('bd_manager_id','int4'),
            ('bd_manager_name','varchar(300)'),
            ('user_id','int4'),
            ('e_card_id','int4'),
            ('main_category','text'),
            ('category_id','int4'),
            ('category_name','text'),
            ('company_reporting_category_id','int4'),
            ('company_reporting_category','varchar(300)'),
            ('currency','varchar(300)'),
            ('payment_type','text'),
            ('status_id','int4'),
            ('status','text'),
            ('refusal_reason_raw','text'),
            ('psp_reference','varchar(300)'),
            ('total_amount','numeric default 0.0'),
            ('promo_code','varchar(300)'),
            ('promo_code_amount','numeric default 0.0'),
            ('credit_charged_amount','numeric default 0.0'),
            ('cash_credit_charged','numeric(20,2) default 0.0'),
            ('cash_charged_amount','numeric default 0.0'),
            ('bonus_credit_amount','numeric default 0.0'),
            ('promo_cashback_granted','numeric(20,2) default 0.0'),
            ('partner_credit_amount','numeric default 0.0'),
            ('used_amount','numeric default 0.0'),
            ('fave_fee_percentage','numeric default 0.0'),
            ('fave_fees','numeric default 0.0'),
            ('gross_profit_utilised','numeric(20,2) default 0.0'),
            ('gross_profit_granted','numeric(20,2) default 0.0'),
            ('details','json'),
            ('created_at','timestamp'),
            ('purchase_date_lt','timestamp'),
            ('cancellation_date_lt','timestamp'),
            ('expiry_date','timestamp'),
            ('updated_at','timestamp'),
            ('updated_at_lt','timestamp'),
            ('gift','bool')
        )


    def create_table(self, table = None):
        
        if table is None:
            table = self

        sql = Query.create_table(table)
        
        for column in self.__columns__:
            sql.columns(column)
        
        return sql

    # ----------------------------------------------------------------------
    # Methods for streaming use case

    def streaming_insert(self, data):
        columns = tuple(data.keys())
        values = tuple(data.values())

        sql = (
            PostgreSQLQuery
            .into(self)
            .insert(values)
            .columns(columns)
            .on_conflict('id')
            .do_nothing()
        )

        return sql
    

    def streaming_update(self, data, columns = None):
        
        if columns is None:
            columns = list(data.keys())

        id = data.pop('id')

        sql = (
            Query
            .update(self)
            .where(self.id == id)
            .set('dwh_updated_at_myt', Timezone('Asia/Taipei', Now()))
        )

        if 'gross_profit_utilised' in columns:
            columns.remove('gross_profit_utilised')
            gross_profit_utilised = data.pop('gross_profit_utilised')
            sql = sql.set(
                'gross_profit_utilised',
                gross_profit_utilised + Field('cash_credit_charged')
            )
        
        if 'gross_profit_granted' in columns:
            columns.remove('gross_profit_granted')
            gross_profit_granted = data.pop('gross_profit_granted')
            sql = sql.set(
                'gross_profit_granted',
                gross_profit_granted - Field('promo_cashback_granted')
            )

        if 'cash_credit_charged' in columns:
            columns.remove('cash_credit_charged')
            cash_credit_charged = data.pop('cash_credit_charged')
            sql = (
                sql
                .set(
                    'cash_credit_charged',
                    Field('cash_credit_charged') + cash_credit_charged
                )
                .set(
                    'gross_profit_utilised', 
                    Field('gross_profit_utilised') + cash_credit_charged
                )
            )
        
        if 'promo_cashback_granted' in columns:
            columns.remove('promo_cashback_granted')
            promo_cashback_granted = data.pop('promo_cashback_granted')
            sql = (
                sql
                .set(
                    'promo_cashback_granted',
                    Field('promo_cashback_granted') + promo_cashback_granted
                )
                .set(
                    'gross_profit_granted', 
                    Field('gross_profit_granted') - promo_cashback_granted
                )
            )

        if len(columns):
            for col in columns:
                sql = sql.set(col, data[col])
        
        return sql